//+------------------------------------------------------------------+
//|                                                       		 Trace |
//|                             Copyright 2006-2011, www.FXmaster.de |
//|                                                  www.FXmaster.de |
//+------------------------------------------------------------------+
#property copyright "Copyright 2006-2011, www.FXmaster.de"
#property link      "www.FXmaster.de"

#include "Tree.mqh"

class CTraceView; //  
//------------------------------------------------------------------	class CTraceCtrl
class CTraceCtrl
{
public: 
	CTreeCtrl *m_stack; //  
	CTreeCtrl *m_info; //  
	CTreeCtrl *m_file; //   
	CTreeCtrl *m_class; //   
	CTraceView* m_traceview; //    
	
	CNode* m_cur; //    
	CTraceCtrl() { Create(); Reset(); } //  
	~CTraceCtrl() { delete m_stack; delete m_info; delete m_file; delete m_class; } //  
	void Create(); //  
	void In(string apath, string afile, int aline, string atext, string adesc, int aid); //     
	void Out(int aid); //    
	bool StepBack(); //        (  )
	void Reset() { m_cur=m_stack.m_root; m_stack.Reset(); m_file.Reset(); m_class.Reset(); } //   
	void Clear() { m_cur=m_stack.m_root; m_stack.Clear(); m_file.Clear(); m_class.Clear(); } //   

public:
	void AddWatch(string name, string val); //     
	void Break(); //   
};
//------------------------------------------------------------------	CTraceCtrl
void CTraceCtrl::Create()
{
	m_stack=new CTreeCtrl; m_stack.m_root.m_text="TRACE"; //   
	m_info=new CTreeCtrl; m_info.m_root.m_text="INFO"; //   
	m_file=new CTreeCtrl; m_file.m_root.m_text="FILE"; //   
	m_class=new CTreeCtrl; m_class.m_root.m_text="CLASS"; //   
	m_cur=m_stack.m_root; //    
}
//------------------------------------------------------------------	In
void CTraceCtrl::In(string apath, string afile, int aline, string atext, string adesc, int aid)
{	
	if (NIL(m_stack) || NIL(m_cur)) return; //  
	if (aid>=0) Out(aid); //    ,    
	//      
	int i, size=ArraySize(m_cur.m_next); //    
	bool b=false, ident;
	for (i=0; i<size; i++) //       
	{
		if (NIL(m_cur.m_next[i])) continue; //  
		if (m_cur.m_next[i].m_text==atext) { b=true; break; } //  ,   
	}
	ident=(m_cur.m_file==afile && m_cur.m_line==aline && m_cur.m_path==apath);
	
	if (b) 
	{
		m_cur=m_cur.m_next[i]; //    
		m_cur.m_desc=adesc; //  
	}
	else if (!ident)//        ,   
	{	
		ArrayResize(m_cur.m_next, size+1); //  
		m_cur.m_next[size]=new CNode;  //  
		if (NIL(m_cur.m_next[size])) return; // 
		m_stack.m_maxid++; //   
		m_cur.m_next[size].m_prev=m_cur; //    
		m_cur=m_cur.m_next[size]; //      
		m_cur.m_id=m_stack.m_maxid;
		m_cur.m_text=atext; 
		//  
		string afunc=atext, aclass="__base__"; int k=StringFind(atext, "::");
		if (k>0) { aclass=StringSubstr(atext, 0, k); afunc=StringSubstr(atext, k+2); }
		m_cur.m_path=apath;
		m_cur.m_file=afile;
		m_cur.m_line=aline;
		m_cur.m_func=afunc;
		m_cur.m_class=aclass;
		m_cur.m_desc=adesc;
		//   
		m_stack.SortBy(TG_TEXT, true);
		m_file.Clear();  m_stack.GroupBy(TG_FILE, m_file); m_file.SortBy(TG_TEXT, true);
		m_class.Clear(); m_stack.GroupBy(TG_CLASS, m_class); m_class.SortBy(TG_TEXT, true);
	}
	m_cur.m_uses++; 
	m_cur.m_tick0=GetTickCount(); m_cur.m_last=TimeLocal(); //      
}
//------------------------------------------------------------------	Out
void CTraceCtrl::Out(int aid)
{
	if (aid<0) { StepBack(); return; }
	CNode *node=m_stack.FindNode(aid); if (NIL(node)) return; //    id
	while (m_cur!=node) if (!StepBack()) break; //   ,    
	if (m_cur!=node) m_cur=node; //    ,  
	if (NIL(m_cur)) m_cur=node; //    ,  
}
//------------------------------------------------------------------	StepBack
bool CTraceCtrl::StepBack()
{
	if (NIL(m_stack.m_root)) { Print("-err Out root"); return(false); } //  
	if (NIL(m_cur)) { m_cur=m_stack.m_root; return(false); } //  
	if (m_cur.m_tick0>0) m_cur.m_tick+=GetTickCount()-m_cur.m_tick0; m_cur.m_tick0=0; //  
	if (m_cur==m_stack.m_root) return(false); //    
	if (NIL(m_cur.m_prev)) { Print("-err Out "+m_cur.m_text+"("+(string)m_cur.m_id+")"); return(false); } //  
	m_cur=m_cur.m_prev; //    
	return(true);
}
//------------------------------------------------------------------	AddWatch
void CTraceCtrl::AddWatch(string name, string val) //  Watch-   
{
	if (!NIL(m_trace) && !NIL(m_trace.m_cur)) m_trace.m_cur.AddWatch(name, val);
}
//------------------------------------------------------------------	Break
void CTraceCtrl::Break() //     
{
	if (NIL(m_traceview)) return; //  
	m_stack.BreakBy(TG_ALL, NULL, false); //   m_break   
	m_cur.m_break=true; //    
	m_traceview.m_viewstack.m_sid=m_cur.m_id; //    
	m_stack.ExpandBy(TG_UP, m_cur, true, m_cur); //   ,  
	m_traceview.Draw(); //  
	string name=m_traceview.m_viewstack.m_name+string(m_cur.m_id)+".dbg"; //    BREAK
	bool state=ObjectGetInteger(m_traceview.m_chart, name, OBJPROP_STATE);
	while(!state) //    ,  
	{
		Sleep(1000); //  
		state=ObjectGetInteger(m_traceview.m_chart, name, OBJPROP_STATE);  //   
		if (!m_traceview.IsOpenView()) break; //   ,  
		m_traceview.Draw(); //   
	}
	m_cur.m_break=false; //  
	m_traceview.Draw(); //  
}

//------------------------------------------------------------------	CIn
class CIn
{
public:
	void In(string apath, string afile, int aline, string afunc, string adesc)
	{
		if (NIL(m_trace)) return; //   ,  
		if (NIL(m_trace.m_stack)) return;
		if (NIL(m_trace.m_stack.m_root)) return;
		if (NIL(m_trace.m_cur))  m_trace.m_cur=m_trace.m_stack.m_root;
		m_trace.In(apath, afile, aline, afunc, adesc, -1); //   
	}
	void ~CIn() { if (!NIL(m_trace)) m_trace.Out(-1); } //  
};
//  
#define _IN(desc)		CIn _in; _in.In(__PATH__, __FILE__, __LINE__, __FUNCTION__, string(desc))
#define _IN1(desc)	CIn _in1; _in1.In(__PATH__, __FILE__, __LINE__, __FUNCTION__, string(desc))
#define _IN2(desc)	CIn _in2; _in2.In(__PATH__, __FILE__, __LINE__, __FUNCTION__, string(desc))
#define _IN3(desc)	CIn _in3; _in3.In(__PATH__, __FILE__, __LINE__, __FUNCTION__, string(desc))
#define _IN4(desc)	CIn _in4; _in4.In(__PATH__, __FILE__, __LINE__, __FUNCTION__, string(desc))
#define _IN5(desc)	CIn _in5; _in5.In(__PATH__, __FILE__, __LINE__, __FUNCTION__, string(desc))

#define _WATCH(w, v) 		if (!NIL(m_trace) && !NIL(m_trace.m_cur)) m_trace.m_cur.AddWatch(w, string(v));


#include "View\TraceView.mqh" //   

extern CTraceCtrl* m_trace; //   
