//+------------------------------------------------------------------+
//|                                                   		 TraceView |
//|                             Copyright 2006-2011, www.FXmaster.de |
//|                                                  www.FXmaster.de |
//+------------------------------------------------------------------+
#property copyright "Copyright 2006-2011, www.FXmaster.de"
#property link      "www.FXmaster.de"
#property version   "1.00"

#include "..\Trace.mqh"
#include "TreeView.mqh"

//------------------------------------------------------------------	class CTreeView
class CTraceView: public CView
{
//---  
public:
	CTraceView() { }; // 
	~CTraceView() { Deinit(); } // 
	void Deinit(); //   
	void Create(long chart); //    

//---   
public:
	int m_hagent; //  -   
	CTraceCtrl* m_trace; //    
	CTreeView* m_viewstack; //    
	CTreeView* m_viewinfo; //     
	CTreeView* m_viewfile; //        
	CTreeView* m_viewclass; //        
	void OnTimer(); //  
	void OnChartEvent(const int , const long& , const double& , const string& ); //  

//---  
public:
	void Draw(); //  
	void DeleteView(); //  

	void UpdateInfoTree(CNode *node, bool bclear); //     
	string TimeSeparate(long time); //      
};
//------------------------------------------------------------------	~CTraceView
void CTraceView::Deinit()
{ _IN("");
	EventKillTimer();
	ObjectsDeleteAll2(m_chart, 0, -1, m_sID);
	delete m_viewclass;
	delete m_viewfile;
	delete m_viewinfo;
	delete m_viewstack;
	ChartIndicatorDelete(m_chart, 0, "!TraceAgent");
	m_hagent=INVALID_HANDLE;
	ChartClose(m_chart);
}
//------------------------------------------------------------------	Create
void CTraceView::Create(long chart)
{ _IN("");
	m_chart=ChartOpen(Symbol(), Period()); //     
	
	//         
	m_hagent=iCustom(Symbol(), Period(), "!TraceAgent", chart);
	if (m_hagent==INVALID_HANDLE) return;
	ChartIndicatorAdd(m_chart, 0, m_hagent);
	
	HideChart(m_chart, LightSteelBlue); //     
	int heigh=(int)ChartGetInteger(m_chart, CHART_HEIGHT_IN_PIXELS)-m_dy-5; //   
	int width=(int)ChartGetInteger(m_chart, CHART_WIDTH_IN_PIXELS)-10; //   
	//         
	double k=1.7, kx=width/(k+2);
	int dh=int(heigh/2.5);
	int x=5, y=m_dy+5, dx=int(k*kx);
	//  
	m_viewstack=new CTreeView; m_viewstack.m_tree=m_trace.m_stack; //    
	m_viewstack.Create(m_chart, "viewstack", 0, Black, White, DarkBlue, x, y, dx, heigh-dh-3, 0, 8, "Tahoma"); //  
	m_viewstack.m_tree.SortBy(TG_TEXT, true);
	m_viewstack.Draw();
	//  
	m_viewinfo=new CTreeView; m_viewinfo.m_tree=m_trace.m_info; //    
	m_viewinfo.m_bProperty=true;
	m_viewinfo.Create(m_chart, "viewinfo", 0, Black, White, DarkBlue, x, y+heigh-dh, dx, dh, 0, 8, "Tahoma"); //  
	m_viewinfo.Draw();
	//   
	x+=dx+3; dx=(int)kx;
	m_viewclass=new CTreeView; m_viewclass.m_tree=m_trace.m_class; //    
	m_viewclass.Create(m_chart, "viewclass", 0, Black, White, DarkBlue, x, y, dx, heigh, 0, 8, "Tahoma"); //  
	m_viewclass.m_tree.SortBy(TG_TEXT, true);
	m_viewclass.Draw();
	//   
	x+=dx+3; dx=(int)kx;
	m_viewfile=new CTreeView; m_viewfile.m_tree=m_trace.m_file; //    
	m_viewfile.Create(m_chart, "viewfile", 0, Black, White, DarkBlue, x, y, dx, heigh, 0, 8, "Tahoma"); //  
	m_viewfile.m_tree.SortBy(TG_TEXT, true);
	m_viewfile.Draw();
	
	EventSetTimer(2);
	ChartRedraw(m_chart);
}
//------------------------------------------------------------------	OnTimer
void CTraceView::OnTimer()
{ _IN("");
	_WATCH("time", TTS(TimeLocal()));
	Draw(); //  
}
//------------------------------------------------------------------	OnChartEvent
void CTraceView::OnChartEvent(const int aid, const long& lparam, const double& dparam, const string& sparam)
{ _IN("");
	if (aid<CHARTEVENT_CUSTOM) return; //     
	int id=aid-CHARTEVENT_CUSTOM;
	_WATCH("Event", EnumToString(ENUM_CHART_EVENT(id)));
	
	//    
	if (id==CHARTEVENT_OBJECT_CLICK)
	{ _IN2("");
		if (StringFind(sparam, "viewstack")>=0) //   
		{
			int btn=m_viewstack.OnClick(sparam); 
			if (btn==BTN_CLEAR) 
			{
				m_viewclass.DeleteView(); m_viewclass.m_tree.Clear(); m_viewclass.Draw();
				m_viewfile.DeleteView(); m_viewfile.m_tree.Clear(); m_viewfile.Draw();
				m_viewinfo.DeleteView();
			}
			if (m_viewstack.m_sid<=0) return;
			CNode *node=m_viewstack.m_tree.FindNode(m_viewstack.m_sid);	if (NIL(node)) return;
			UpdateInfoTree(node, true);
			m_viewclass.m_tree.SelectBy(TG_ALL, NULL, false); m_viewclass.m_tree.CheckBy(TG_ALL, NULL, false);
			m_viewclass.m_tree.SelectBy(TG_CLASS|TG_FUNC, node, true); m_viewclass.m_tree.CheckBy(TG_CLASS|TG_FUNC, node, true);
			m_viewclass.m_sid=-1; m_viewclass.Draw();
			
			m_viewfile.m_tree.SelectBy(TG_ALL, NULL, false); m_viewfile.m_tree.CheckBy(TG_ALL, NULL, false); 
			m_viewfile.m_tree.SelectBy(TG_FILE|TG_FUNC, node, true); m_viewfile.m_tree.CheckBy(TG_FILE|TG_FUNC, node, true);
			m_viewfile.m_sid=-1; m_viewfile.Draw();
			ChartRedraw(m_chart);
		}
		if (StringFind(sparam, "viewclass")>=0) //   
		{
			int btn=m_viewclass.OnClick(sparam); 
			if (btn==BTN_CLEAR) 
			{	
				m_viewstack.m_tree.GroupBy(TG_CLASS, m_viewclass.m_tree); m_viewclass.m_tree.SortBy(TG_TEXT, true);
				m_viewclass.Draw();
			}
			m_viewinfo.DeleteView();
			if (m_viewclass.m_sid<=0) return;
			CNode *node=m_viewclass.m_tree.FindNode(m_viewclass.m_sid);	if (NIL(node)) return;
			m_viewstack.m_tree.SelectBy(TG_ALL, NULL, false); m_viewstack.m_tree.CheckBy(TG_ALL, NULL, false);
			m_viewstack.m_tree.SelectBy(TG_CLASS|TG_FUNC, node, true); m_viewstack.m_tree.CheckBy(TG_CLASS|TG_FUNC, node, true);
			m_viewstack.m_sid=-1; m_viewstack.Draw();

			m_viewfile.m_tree.SelectBy(TG_ALL, NULL, false); m_viewfile.m_tree.CheckBy(TG_ALL, NULL, false);
			m_viewfile.m_tree.SelectBy(TG_FILE|TG_FUNC, node, true); m_viewfile.m_tree.CheckBy(TG_FILE|TG_FUNC, node, true);
			m_viewfile.m_sid=-1; m_viewfile.Draw();
			ChartRedraw(m_chart);
		}
		if (StringFind(sparam, "viewfile")>=0) //   
		{
			int btn=m_viewfile.OnClick(sparam); 
			if (btn==BTN_CLEAR) 
			{ 
				m_viewstack.m_tree.GroupBy(TG_FILE, m_viewfile.m_tree); m_viewfile.m_tree.SortBy(TG_TEXT, true); 
				m_viewfile.Draw();
			}
			m_viewinfo.DeleteView();
			if (m_viewfile.m_sid<=0) return;
			CNode *node=m_viewfile.m_tree.FindNode(m_viewfile.m_sid);	if (NIL(node)) return;
			m_viewstack.m_tree.SelectBy(TG_ALL, NULL, false); m_viewstack.m_tree.CheckBy(TG_ALL, NULL, false);
			m_viewstack.m_tree.SelectBy(TG_FILE|TG_FUNC, node, true); m_viewstack.m_tree.CheckBy(TG_FILE|TG_FUNC, node, true);
			m_viewstack.m_sid=-1; m_viewstack.Draw();

			m_viewclass.m_tree.SelectBy(TG_ALL, NULL, false); m_viewclass.m_tree.CheckBy(TG_ALL, NULL, false);
			m_viewclass.m_tree.SelectBy(TG_FILE|TG_FUNC, node, true); m_viewclass.m_tree.CheckBy(TG_FILE|TG_FUNC, node, true);
			m_viewclass.m_sid=-1; m_viewclass.Draw();
			ChartRedraw(m_chart);
		}
		if (StringFind(sparam, "viewinfo")>=0) //   
		{
			m_viewinfo.OnClick(sparam); ChartRedraw(m_chart);
		}
	}
	if (id==CHARTEVENT_CHART_CHANGE) 
	{
		Draw(); //    
	}
}
//------------------------------------------------------------------	Draw
void CTraceView::Draw()
{ _IN("");
	int heigh=(int)ChartGetInteger(m_chart, CHART_HEIGHT_IN_PIXELS)-m_dy-5; //  
	int dh=int(heigh/2.5);
	m_viewstack.m_dy=heigh-dh-3; m_viewinfo.m_dy=dh; m_viewinfo.m_y=m_viewstack.m_y+heigh-dh;
	m_viewclass.m_dy=heigh; m_viewfile.m_dy=heigh;

	int width=(int)ChartGetInteger(m_chart, CHART_WIDTH_IN_PIXELS)-10; //   
	double k=1.7, kx=width/(k+2);
	int x=5, dx=int(k*kx); m_viewstack.m_dx=dx; m_viewinfo.m_dx=dx; 
	x+=dx+3; dx=int(kx); m_viewclass.m_x=x; m_viewclass.m_dx=dx; 
	x+=dx+3; dx=int(kx); m_viewfile.m_x=x; m_viewfile.m_dx=dx;
	m_viewstack.Draw(); m_viewclass.Draw(); m_viewfile.Draw(); //  
	CNode *node=m_viewstack.m_tree.FindNode(m_viewstack.m_sid);
	UpdateInfoTree(node, false);
	ChartRedraw(m_chart);
}
//------------------------------------------------------------------	Draw
void CTraceView::DeleteView()
{ _IN("");
	m_viewstack.DeleteView(); m_viewinfo.DeleteView();
	m_viewclass.DeleteView(); m_viewfile.DeleteView(); //  
}

//------------------------------------------------------------------	UpdateInfoTree
void CTraceView::UpdateInfoTree(CNode *node, bool bclear)
{
	if (NIL(node)) return;
	if (NIL(m_viewinfo.m_tree.m_root)) return;
	
	//1.        
	CNode *root=m_viewinfo.m_tree.m_root;
	root.m_path=node.m_path; //  
	root.m_line=node.m_line; //    
	
	int id, id2;
	//2.    
	id=m_viewinfo.m_tree.AddNode(0, "Run-time ");
	id2=m_viewinfo.m_tree.AddNode(id, " "); m_viewinfo.m_tree.SetDataBy(TG_DESC, id2, string(node.m_uses));
	id2=m_viewinfo.m_tree.AddNode(id, " ");  m_viewinfo.m_tree.SetDataBy(TG_DESC, id2, TimeSeparate(node.m_tick));
	id2=m_viewinfo.m_tree.AddNode(id, " "); m_viewinfo.m_tree.SetDataBy(TG_DESC, id2, TTS(node.m_last));

	id=m_viewinfo.m_tree.AddNode(0, "");
	id2=m_viewinfo.m_tree.AddNode(id, ""); m_viewinfo.m_tree.SetDataBy(TG_DESC, id2, node.m_file);
	id2=m_viewinfo.m_tree.AddNode(id, " "); m_viewinfo.m_tree.SetDataBy(TG_DESC, id2, string(node.m_line));
	id2=m_viewinfo.m_tree.AddNode(id, ""); m_viewinfo.m_tree.SetDataBy(TG_DESC, id2, node.m_class);
	id2=m_viewinfo.m_tree.AddNode(id, ""); m_viewinfo.m_tree.SetDataBy(TG_DESC, id2, node.m_func);
	id=m_viewinfo.m_tree.AddNode(0, "Watch-");
	if (bclear)
	{
		CNode *w=m_viewinfo.m_tree.FindNode(id); //   watch
		m_viewinfo.DeleteView(w, false); //   
		m_viewinfo.m_tree.Clear(w); //   
	}
	//  watch
	int n=ArraySize(node.m_watch);
	for (int i=0; i<n; i++)
	{
		id2=m_viewinfo.m_tree.AddNode(id, node.m_watch[i].m_name); m_viewinfo.m_tree.SetDataBy(TG_DESC, id2, node.m_watch[i].m_val);
	}
	m_viewinfo.Draw();
}
//------------------------------------------------------------------	TimeSeparate
string CTraceView::TimeSeparate(long time)
{
	long sec=1000; long min=sec*60; long hour=min*60; long day=hour*24;
	string txt="";
	long d=long(time/day); if (d>0) txt+=string(d)+"  ";
	long h=long(time/hour)-d*day; if (h>9) txt+=string(h)+":"; else txt+="0"+string(h)+":";
	long m=long(time/min)-(h*hour+d*day); if (m>9) txt+=string(m)+":"; else txt+="0"+string(m)+":";
	long s=long(time/sec)-(m*min+h*hour+d*day); if (s>9) txt+=string(s)+"."; else txt+="0"+string(s)+".";
	long ms=time-(s*sec+m*min+h*hour+d*day); txt+=string(ms);
	return(txt);
}

extern CTraceView* m_traceview; //    
